// Andrei Grigorean - 100 puncte
#include <cassert>
#include <cstdio>

#include <algorithm>
#include <set>
#include <vector>
using namespace std;

const int MAX_LEN = 1000000000;
const int MAX_N = 100005;
const int INF = 0x3f3f3f3f;

struct Edge {
  int x, y;
};

struct Event {
  int time;
  int type;
  int index;

  bool operator < (const Event &other) const {
    return time < other.time;
  }
};

int length, n;
Edge edges[MAX_N];

int st[MAX_N];
vector<Event> events;
int aint[4 * MAX_N];
long long ans[MAX_N];

// Debug stuff
set<int> usedCities;

void read() {
  assert(freopen("minuni.in", "r", stdin) != NULL);
  assert(freopen("minuni.out", "w", stdout) != NULL);

  assert(scanf("%d %d", &length, &n) == 2);
  assert(1 <= length && length <= MAX_LEN);
  assert(1 <= n && n <= MAX_N);

  for (int i = 1; i <= n; ++i) {
    assert(scanf("%d %d", &edges[i].x, &edges[i].y) == 2);
    assert(1 <= edges[i].x && edges[i].x <= length);
    assert(1 <= edges[i].y && edges[i].y <= length);

    assert(usedCities.find(edges[i].x) == usedCities.end());
    usedCities.insert(edges[i].x);
    assert(usedCities.find(edges[i].y) == usedCities.end());
    usedCities.insert(edges[i].y);
  }
}

void update(int nod, int st, int dr, int a, int b) {
  if (st == dr) {
    aint[nod] = b;
    return;
  }
  int mij = (st + dr) / 2;
  if (a <= mij)
    update(nod * 2, st, mij, a, b);
  else
    update(nod * 2 + 1, mij + 1, dr, a, b);

  aint[nod] = max(aint[nod * 2], aint[nod * 2 + 1]);
}

int query(int nod, int st, int dr, int a, int b) {
  if (a <= st && dr <= b)
    return aint[nod];

  int ans = 0;
  int mij = (st + dr) / 2;

  if (a <= mij)
    ans = max(ans, query(nod * 2, st, mij, a, b));
  if (b > mij)
    ans = max(ans, query(nod * 2 + 1, mij + 1, dr, a, b));

  return ans;
}

void solve() {
  // Find events
  for (int i = 1; i <= n; ++i) {
    events.push_back((Event){edges[i].x, 0, i});
    events.push_back((Event){edges[i].y, 1, i});
  }
  sort(events.begin(), events.end());

  // Do the monkey
  for (int i = 0; i < int(events.size()); ++i)
    if (events[i].type == 0) {
      int edge = events[i].index;

      int stackIndex = query(1, 1, n, 1, edge);
      int father = stackIndex ? st[stackIndex] : 0;

      if (father) {
        long long a = edges[father].x;
        long long b = edges[events[i].index].x;
        long long c = edges[events[i].index].y;
        long long d = edges[father].y;
        ans[events[i].index] = b * (d - c + 1) - a +
                              (b - a + 1) * (length - c + 1) - (length - d  +1) -
                              (b - a + 1) * (d - c + 1) + 1;

      } else {
        long long a = edges[edge].x;
        long long b = edges[edge].y;
        ans[events[i].index] = a * (length - b + 1);
      }

      st[++st[0]] = events[i].index;
      update(1, 1, n, events[i].index, st[0]);
    } else {
      int index = st[st[0]];
      --st[0];
      assert(st[0] >= 0);
      update(1, 1, n, index, 0);
    }
  assert(st[0] == 0);
}

void write() {
  for (int i = 1; i <= n; ++i)
    printf("%lld\n", ans[i]);
}

int main() {
  read();
  solve();
  write();
}
