// Mugurel Ionut Andreica - Bucuresti

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#define NMAX 512
#define VMIN -666666666

#define myabs(a) ((a) >= 0 ? (a) : (-(a)))
#define max(a, b) ((a) > (b) ? (a) : (b))
#define min(a, b) ((a) < (b) ? (a) : (b))

#define SMART_SOL 1
#define PRINT_DEBUG 0

int P[NMAX][NMAX], pmax[NMAX][NMAX], cmax[NMAX][NMAX];
int i, j, k, N, T, Q;

void readInputData()
{
	freopen("v2d.in", "r", stdin);
	scanf("%d %d %d", &N, &T, &Q);

	for (i = 0; i < N; i++)
		for (j = 0; j < N; j++)
			scanf("%d", &P[i][j]);
}

void smartSol(void)
{
	int pi, pj, idir, istart, ifin, jdir, jstart, jfin, vcur, vcur2, t;

	if (PRINT_DEBUG)
		freopen("v2d.log.txt", "w", stderr);

	for (t = 1; t <= T; t++)
	{
		for (i = 0; i < N; i++)
			for (j = 0; j < N; j++)
				pmax[i][j] = 0;

		for (pi = 0; pi <= 1; pi++)
			for (pj = 0; pj <= 1; pj++)
			{
				idir = 2 * (1 - pi) - 1;
				istart = (N - 1) * pi;
				ifin = (N - 1) * (1 - pi) + idir;

				jdir = 2 * (1 - pj) - 1;
				jstart = (N - 1) * pj;
				jfin = (N - 1) * (1 - pj) + jdir;

				for (i = istart; i != ifin; i += idir)
					for (j = jstart; j != jfin; j += jdir)
						{
							vcur = VMIN;

							if (i != istart)
								vcur = max(vcur, cmax[i - idir][j]);

							if (j != jstart)
								vcur = max(vcur, cmax[i][j - jdir]);

							vcur2 = vcur - idir * i - jdir * j;
							pmax[i][j] = max(vcur2, pmax[i][j]);

							vcur = max(vcur, P[i][j] + idir * i + jdir * j);
							cmax[i][j] = vcur;
						}
			}

		if (PRINT_DEBUG)
		{
			fprintf(stderr, "%d\n", t);
			for (i = 0; i < N; i++)
			{
				for (j = 0; j < N; j++)
					fprintf(stderr, "%d ", pmax[i][j]);
				fprintf(stderr, "\n");
			}
		}
			
		for (i = 0; i < N; i++)
			for (j = 0; j < N; j++)
				P[i][j] = 1 + t + ((P[i][j] + pmax[i][j] * t) % Q);
	}
}

void sillySol(void)
{
	int ip, jp, t, p;

	for (t = 1; t <= T; t++)
	{
		for (i = 0; i < N; i++)
			for (j = 0; j < N; j++)
				{
					pmax[i][j] = 0;

					for (ip = 0; ip < N; ip++)
						for (jp = 0; jp < N; jp++)
								if (ip != i || jp != j)
								{
									p = P[ip][jp] - myabs(i - ip) - myabs(j - jp);
									pmax[i][j] = max(pmax[i][j], p);
								}
				}

		for (i = 0; i < N; i++)
			for (j = 0; j < N; j++)
				P[i][j] = 1 + t + ((P[i][j] + pmax[i][j] * t) % Q);
	}
}

void writeOutputData(void)
{
	int sum = 0;

	freopen("v2d.out", "w", stdout);

	for (i = 0; i < N; i++)
		for (j = 0; j < N; j++)
			sum = (sum + P[i][j]) % Q;

	printf("%d\n", sum);
	fprintf(stderr, "%d\n", sum);
}

int main()
{
	int tstart = clock();

	readInputData();
	
	if (SMART_SOL)
		smartSol();
	else
		sillySol();

	writeOutputData();

	fprintf(stderr, "Duration=%.4lf sec\n", (double) (clock() - tstart) / CLOCKS_PER_SEC);

	return 0;
}
