/*
 * Author: Cosmin Gheorghe
 * Time Complexity: O(3^N)
 */

#include <stdio.h>
#include <math.h>
#include <vector>
#include <algorithm>
using namespace std;

#define MP make_pair
#define ff first
#define ss second
#define eps 1e-8

#define NMAX 410
const double pi = acos(-1);

int N;

double xp[NMAX], yp[NMAX];

vector <pair<double, double> > C;

double din[NMAX * NMAX];
int p_beg[NMAX * NMAX], p_end[NMAX * NMAX];
int pred[NMAX * NMAX];

void baga_cerc(double x, double y)
{
	C.push_back(MP(x, y));
}

void baga_cerc(double x1, double y1, double x2, double y2)
{
	if (x1 == x2) return;

	double xm = (x1 + x2) * 0.5;
	double ym = (y1 + y2) * 0.5;
	double ap = x1 - x2;
	double bp = y1 - y2;

	double xf = (ap * xm + bp * ym) / ap;
	double rf = sqrt((xf - x1) * (xf - x1) + y1 * y1);

	C.push_back(MP(xf, rf));
}

int in_cerc(double x, double y, double c, double r)
{
	return (x - c) * (x - c) + y * y <= r * r + eps;
}

int merge_cerc(double c1, double r1, double c2, double r2)
{
	if (in_cerc(c1, r1, c2, r2) || in_cerc(c2, r2, c1, r1)) return 0;

	int i;

	for (i = 1; i <= N; i++)
		if (c1 <= xp[i] && xp[i] <= c2)
			if (!in_cerc(xp[i], yp[i], c1, r1) && !in_cerc(xp[i], yp[i], c2, r2))
				return 0;

	return 1;
}

int merge_raza(int vec[], double r)
{
	double xbeg = -1e10, xend = 1e10;
	
	for (int i = 1; i <= vec[0]; i++) {
		double x = xp[vec[i]], y = yp[vec[i]];

		if (y > r) return 0;

		double dst = sqrt(r * r - y * y);
		double x1 = x - dst, x2 = x + dst;

		xbeg = max(xbeg, x1);
		xend = min(xend, x2);
	}

	return xbeg <= xend;
}

double din3n[1 << 15];
double rc[1 << 15];

void solve_3n()
{
	int i, j, k;

	int vec[30];

	// calculez raza cercurilor tuturor submultimilor 2^n
	
	for (i = 1; i < (1 << N); i++) {
		vec[0] = 0;

		for (j = 1, k = 1; j <= i; j <<= 1, k++)
			if (j & i) vec[++vec[0]] = k;
		
		double r = 0, step = 1 << 20;

		for (int q = 1; q <= 40; q++) {
			if (!merge_raza(vec, r + step)) r += step;

			step *= 0.5;
		}

		rc[i] = r;

//		printf("%d - %lf\n", i, rc[i]);
	}

	din3n[0] = 0;

	for (i = 1; i < (1 << N); i++) {
		din3n[i] = 1e+10;

		for (j = 1; j < (1 << N); j++)
			if ((i & j) == j) din3n[i] = min(din3n[i], din3n[i ^ j] + rc[j] * rc[j]);
	}

	printf("%0.4lf\n", din3n[(1 << N) - 1] * pi);
}

pair<double, double> pct[NMAX];

int main()
{
	int i, j;

	freopen("acerc.in", "r", stdin);
	freopen("acerc.out", "w", stdout);

	scanf("%d", &N);

	for (i = 1; i <= N; i++) {
		scanf("%lf %lf", &xp[i], &yp[i]);
		pct[i] = MP(xp[i], yp[i]);
	}

	sort(pct + 1, pct + N + 1);

	for (i = 1; i <= N; i++) xp[i] = pct[i].ff, yp[i] = pct[i].ss;

	for (i = 1; i <= N; i++) {
		baga_cerc(xp[i], yp[i]);
		for (j = i + 1; j <= N; j++) {
			baga_cerc(xp[i], yp[i], xp[j], yp[j]);
		}
	}

	sort(C.begin(), C.end());

	solve_3n();

	return 0;
}

