/*
 * Author: Andrei Grigorean
 * Time complexity: O(N^2 * big_numbers)
 */

#include <cassert>
#include <cstdio>
#include <cstring>

const int MAX_N = 305;
const int MAX_DIGIT = 700;
const int BASE = 10;

int n, m;
char s[MAX_DIGIT];
int k[MAX_DIGIT];

int pos[MAX_N], pos2[MAX_N];
bool used[MAX_N];
int ways[MAX_DIGIT], ways2[MAX_DIGIT], diff[MAX_DIGIT];

void read() {
    assert(freopen("bubblesort.in", "r", stdin) != NULL);
    assert(freopen("bubblesort.out", "w", stdout) != NULL);

    assert(scanf("%d %d %s", &n, &m, s) == 3);
    assert(1 <= n && n <= 300);
    assert(1 <= m && m <= n);

    k[0] = strlen(s);
    for (int i = 1; i <= k[0]; ++i)
        k[i] = s[k[0] - i] - '0';
}

void multiply(int A[], int B) {
    if (B == 0) {
        A[0] = 0;
        return;
    }
    int i, t = 0;
    for (i = 1; i <= A[0] || t; ++i, t /= BASE)
        A[i] = (t += A[i] * B) % BASE;
    A[0] = i - 1;
}

void divide(int A[], int B) {
    int i, t = 0;
    for (i = A[0]; i; --i, t %= B)
        A[i] = (t = t * BASE + A[i]) / B;
    for (; A[0] && !A[A[0]]; --A[0]);
}

void atrib(int A[], int B[]) {
    for (int i = 0; i < MAX_DIGIT; ++i)
        A[i] = B[i];
}


void substract(int A[], int B[]) {
    int i, t = 0;
    for (i = 1; i <= A[0]; i++)
        A[i] += (t = (A[i] -= ((i <= B[0]) ? B[i] : 0) + t) < 0) * 10;
    for (; A[0] > 1 && !A[A[0]]; A[0]--);
}

int cmp(int A[], int B[]) {
    if (A[0] != B[0]) return A[0] < B[0] ? -1 : 1;
    for (int i = A[0]; i; --i) {
        if (A[i] < B[i]) return -1;
        if (A[i] > B[i]) return 1;
    }
    return 0;
}

void solve() {
    memset(used, false, sizeof(used));

    for (int i = 1; i <= n; ++i) {
        if (i <= n - m + 1)
            pos[i] = m;
        else
            pos[i] = pos[i - 1] - 1;

        if (i <= n - m + 2)
            pos2[i] = m - 1;
        else
            pos2[i] = pos2[i - 1] - 1;
    }

    ways[0] = ways[1] = 1;
    ways2[0] = ways2[1] = 1;
    for (int i = 1; i <= n; ++i) {
        multiply(ways, pos[i]);
        multiply(ways2, pos2[i]);
    }

    for (int i = 1; i <= n; ++i) {
        for (int j = 1; j <= n; ++j)
            if (!used[j]) {
                if (ways[0]) divide(ways, pos[j]);
                if (ways2[0]) divide(ways2, pos2[j]);

                atrib(diff, ways);
                substract(diff, ways2);

                if (cmp(diff, k) != -1) {
                    printf("%d%s", j, i == n ? "\n" : " ");
                    used[j] = true;
                    break;
                } else {
                    substract(k, diff);
                    --pos[j];
                    --pos2[j];

                    multiply(ways, pos[j]);
                    multiply(ways2, pos2[j]);
                }
            }
    }
}

int main() {
    read();
    solve();
}
