/*
  Author: Tiberiu Savin
  Complexity: N + M log N
*/
#include <cstdio>
#include <vector>
#include <cstring>
#include <cassert>

using namespace std;

#define NMAX 100020
#define INF 0x3f3f3f3f

int N, M;
vector< pair<int, int> > G[NMAX];
int viz[NMAX];
pair<int, int> V[NMAX];
int K[NMAX];
pair< pair<int, int>, int> ARB[3 * NMAX];
pair<int, int> intv[NMAX];
vector<int> Q;
vector< pair<int, int> > ret[NMAX];
int poz;
int p[NMAX];

// Segment Tree structure:
// node = pair< pair<cost, index>, sum>
void update(int nod, int st, int dr, int a, int b, int cst) {
  if (b < a) {
    return;
  }

  if (a <= st && dr <= b) {
    ARB[nod].second += cst;
  } else {
    int mid = (st + dr) / 2;
    if (a <= mid) {
      update(nod * 2, st, mid, a, b, cst);
    }
    if (b > mid) {
      update(nod * 2 + 1, mid + 1, dr, a, b, cst);
    }

    pair<int, int> X, Y;
    X = make_pair(ARB[nod * 2].first.first + ARB[nod * 2]. second, ARB[nod * 2].first.second);
    Y = make_pair(ARB[nod * 2 + 1].first.first + ARB[nod * 2 + 1]. second, ARB[nod * 2 + 1].first.second);
    ARB[nod].first = max(X, Y);
  }
}

pair<int, int> query(int nod, int st, int dr, int a, int b) {
  if (a <= st && dr <= b) {
    return make_pair(ARB[nod].first.first + ARB[nod].second, ARB[nod].first.second);
  } else {
    int mid = (st + dr) / 2;
    pair<int, int> X = make_pair(-INF, 0), Y = make_pair(-INF, 0);
    if (a <= mid) {
      X = query(nod * 2, st, mid, a, b);
    }
    if (b > mid) {
      Y = query(nod * 2 + 1, mid + 1, dr, a, b);
    }

    return max(X, Y);
  }
}

void build(int nod, int st, int dr) {
  if (st == dr) {
    ARB[nod].first = V[st];
  } else {
    int mid = (st + dr) / 2;
    build(nod * 2, st, mid);
    build(nod * 2 + 1, mid + 1, dr);
    ARB[nod] = max(ARB[nod * 2], ARB[nod * 2 + 1]);
  }
}

// Liniarizez arborele
void DF(int nod, int cst) {
  viz[nod] = 1;
  V[++poz] = make_pair(cst, nod);
  intv[nod].first = poz;
  int cnt = 0;
  for (int i = 0; i < G[nod].size(); i++) {
    if (viz[G[nod][i].first]) {
      continue;
    }
    DF(G[nod][i].first, cst + G[nod][i].second);
  }
  intv[nod].second = poz;
}

// Parcurg arborele facand update-urile necesare in arborele de intervale
void compute(int nod) {
  int x = intv[nod].first, y = intv[nod].second;
  viz[nod] = 1;
  for (int i = 1; i <= K[nod]; i++) {
    pair<int, int> ans = query(1, 1, N, 1, N);
    ret[nod].push_back(ans);
    update(1, 1, N, intv[ans.second].first, intv[ans.second].first, -INF);
  }

  for (int i = 0; i < ret[nod].size(); i++) {
    int node = ret[nod][i].second;
    update(1, 1, N, intv[node].first, intv[node].first, INF);
  }

  for (int i = 0; i < G[nod].size(); i++) {
    int son, cst;
    son = G[nod][i].first;
    cst = G[nod][i].second;

    if (viz[son]) {
      continue;
    }

    update(1, 1, N, 1, intv[son].first - 1, cst);
    update(1, 1, N, intv[son].first, intv[son].second, -cst);
    update(1, 1, N, intv[son].second + 1, N, cst);
    compute(son);
    update(1, 1, N, 1, intv[son].first - 1, -cst);
    update(1, 1, N, intv[son].first, intv[son].second, cst);
    update(1, 1, N, intv[son].second + 1, N, -cst);
  }
}

int main() {
  freopen("regat.in", "r", stdin);
  freopen("regat.out", "w", stdout);

  assert(scanf("%d %d ", &N, &M) == 2);
  assert(N <= 100000 && N >= 1);

  for (int i = 1; i < N; i++) {
    int a, b, c;
    assert(scanf("%d %d %d", &a, &b, &c) == 3);
    assert(c <= 10000 && c >= 1);
    G[a].push_back(make_pair(b, c));
    G[b].push_back(make_pair(a, c));
  }

  memset(viz, 0, sizeof(viz));
  poz = 0;
  DF(1, 0);

  for (int i = 1; i <= M; i++) {
    int x;
    assert(scanf("%d ", &x) == 1);
    assert(x > 0 && x <= N);
    Q.push_back(x);
    K[x]++;
  }

  build(1, 1, N);
  memset(viz, 0, sizeof(viz));
  compute(1);

  for (int i = 0; i < Q.size(); i++) {
    printf("%d\n", ret[Q[i]][p[Q[i]]].first);
    p[Q[i]]++;
  }

  return 0;
}
