/*
Tree Square Root
Time Complexity: O(N^3)
*/

#include <stdio.h>
#include <string.h>
#include <vector>

using namespace std;

#define NMAX 350 

int T2[NMAX][NMAX], T[NMAX][NMAX], list[NMAX][NMAX], pref[NMAX][NMAX], cnt[NMAX];
int nv[NMAX], nvc[NMAX][NMAX], viz[NMAX], v[NMAX];
vector<vector<int> > leaf[NMAX], nonleaf[NMAX];
int isleaf[NMAX][NMAX], cntnonleaf[NMAX][NMAX];
int i, j, k, p, q, ii, jj, lgr, lf, N, M, cmax, imax, mapp[2];

void buildtree(int lev)
{
	int i, j, k, ii, jj, p, q, lf, lgr;
	vector<int> l, nl;

	if (cnt[lev] == 1)
		return;

	if (cnt[lev] == 2)
	{
		T[list[lev][1]][list[lev][2]] = T[list[lev][2]][list[lev][1]] = 1;
		return;
	}

	memset(viz, 0, sizeof(viz));

	for (ii = 1; ii <= cnt[lev]; ii++)
	{
		i = list[lev][ii];

		if (viz[i])
			continue;

		// check if i is a leaf in the tree at level lev

		lf = 1;

		for (jj = 1; jj <= cnt[lev]; jj++)
		{
			j = list[lev][jj];

			if (T2[i][j] && nvc[i][j] != nv[i] - 1)
				lf = 0;
		}

		if (!lf)
			continue;

		// i is a leaf in the tree at level lev

		l.clear();
		nl.clear();

		l.push_back(i);
		isleaf[lev][i] = 1;

		for (jj = 1; jj <= cnt[lev]; jj++)
		{
			j = list[lev][jj];

			if (T2[i][j]) // j is a neighbour of i
			{
				viz[j] = 1;

				if (nv[j] == nv[i]) // j is a leaf
				{
					isleaf[lev][j] = 1;
					l.push_back(j);
				}
				else // j is not a leaf
				{
					cntnonleaf[lev][j]++;
					nl.push_back(j);
				}
			}
		}

		leaf[lev].push_back(l);
		nonleaf[lev].push_back(nl);

		fprintf(stderr, "At level %d we found the group of leaves #%d\n", lev, leaf[lev].size());
		fprintf(stderr, "  Leaves:");
		for (jj = 0; jj < l.size(); jj++)
			fprintf(stderr, " %d", l[jj]);
		fprintf(stderr, "\n");
		fprintf(stderr, "  Non-leaves:");
		for (jj = 0; jj < nl.size(); jj++)
			fprintf(stderr, " %d", nl[jj]);
		fprintf(stderr, "\n\n");
	}

	// eliminate all the leaves at this level => update nv and nvc

	for (lgr = 0; lgr < leaf[lev].size(); lgr++)
		for (jj = 0; jj < leaf[lev][lgr].size(); jj++)
		{
			k = leaf[lev][lgr][jj]; // eliminate node #k, because it is a leaf

			for (p = 1; p <= cnt[lev]; p++)
				if (T2[list[lev][p]][k])
				{
					nv[list[lev][p]]--;

					for (q = 1; q <= cnt[lev]; q++)
						if (T2[list[lev][q]][k])
							nvc[list[lev][p]][list[lev][q]]--;
				}
		}

	// build the tree recursively

	if (leaf[lev].size() == 1) // the tree is a star
	{
		// try to find the center

		if (lev > 0)
		{
			cmax = 1;
			imax = 0;

			for (ii = 1; ii <= cnt[lev]; ii++)
			{
				i = list[lev][ii];

				if (cntnonleaf[lev - 1][i] > cmax) // we've got a potential center
				{
					cmax = cntnonleaf[lev - 1][i];
					imax = ii;
				}
			}

			if (imax > 0)
				ii = imax, i = list[lev][imax];
		}
		else
			ii = cnt[lev] + 1;

		if (ii == cnt[lev] + 1) // don't care which node is the center
			ii = 1;

		isleaf[lev][list[lev][ii]] = 0;

		for (jj = 1; jj <= cnt[lev]; jj++)
			if (jj != ii)
				T[list[lev][ii]][list[lev][jj]] = T[list[lev][jj]][list[lev][ii]] = 1;
	}
	else
	{
		cnt[lev + 1] = 0;

		for (ii = 1; ii <= cnt[lev]; ii++)
		{
			i = list[lev][ii];

			if (!isleaf[lev][i])
				list[lev + 1][++cnt[lev + 1]] = i;
		}

		// fill the preference matrix in case it needs to be used at the next level

		if (cnt[lev + 1] > 2)
		{
			memset(pref, 0, sizeof(pref));

			for (k = 0; k < nonleaf[lev].size(); k++)
				for (ii = 0; ii < nonleaf[lev][k].size(); ii++)
					for (jj = ii + 1; jj < nonleaf[lev][k].size(); jj++)
						pref[nonleaf[lev][k][ii]][nonleaf[lev][k][jj]] =
						pref[nonleaf[lev][k][jj]][nonleaf[lev][k][ii]] = 1;
		}

		// build the tree at level lev + 1

		buildtree(lev + 1);

		// find the right parent for every group of leaves

		if (leaf[lev].size() == 2 && cnt[lev + 1] == 2) // the tree at level lev + 1 is just an edge
		{
			mapp[0] = 1; mapp[1] = 2;

			// check the preferences

			for (k = 0; k < 2; k++)
				for (ii = 0; ii < leaf[lev][k].size(); ii++)
				{
					i = leaf[lev][k][ii];

					if (pref[i][list[lev + 1][3 - mapp[k]]])
					{
						mapp[k] = 3 - mapp[k];
						mapp[1 - k] = 3 - mapp[k];
					}
				}

			for (k = 0; k < 2; k++)
				for (ii = 0; ii < leaf[lev][k].size(); ii++)
				{
					i = leaf[lev][k][ii];
					T[i][list[lev + 1][mapp[k]]] = T[list[lev + 1][mapp[k]]][i] = 1;
				}
		}
		else
		{
			// the fun part :)

			for (lgr = 0; lgr < leaf[lev].size(); lgr++)
			{
				// group of leaves #lgr

				if (nonleaf[lev][lgr].size() == 2) // the parent of this group of leaves is a leaf in the tree at level lev + 1
				{
					for (k = 0; k < 2; k++)
						if (isleaf[lev + 1][nonleaf[lev][lgr][k]])
							break;

					k = nonleaf[lev][lgr][k];

					for (ii = 0; ii < leaf[lev][lgr].size(); ii++)
					{
						i = leaf[lev][lgr][ii];

						T[i][k] = T[k][i] = 1;
					}
				}
				else // the parent of this group of leaves is an inner node in the tree at level k + 1
				{
					// the really fun part :)

					for (ii = 0; ii < nonleaf[lev][lgr].size(); ii++)
					{
						i = nonleaf[lev][lgr][ii];

						// count how many neighbours the potential parent i has in T among the potential parents

						k = 0;

						for (jj = 0; jj < nonleaf[lev][lgr].size(); jj++)
						{
							j = nonleaf[lev][lgr][jj];

							if (T[i][j])
								k++;
						}

						if (k == nonleaf[lev][lgr].size() - 1) // found the parent :) -> it's node i
						{
							for (ii = 0; ii < leaf[lev][lgr].size(); ii++)
							{
								k = leaf[lev][lgr][ii];
								T[i][k] = T[k][i] = 1;
							}							

							break;
						}
					}
				}
			}
		}
	}
}

int main()
{
	int i, j, k;

	freopen("tree2.in", "r", stdin);

	scanf("%d %d", &N, &M);

	memset(T2, 0, sizeof(T2));

	while (M--)
	{
		scanf("%d %d", &i, &j);
		T2[i][j] = T2[j][i] = 1;
	}

	memset(T, 0, sizeof(T));

	for (i = 1; i <= N; i++)
		list[0][i] = i;

	cnt[0] = N;

	memset(nvc, 0, sizeof(nvc));
	memset(nv, 0, sizeof(nv));

	for (i = 1; i <= N; i++)
	{
		for (j = 1; j <= N; j++)
		{
			if (T2[i][j])
				nv[i]++;

			for (k = 1; k <= N; k++)
				if (T2[i][k] && T2[j][k])
					nvc[i][j]++;
		}
	}

	memset(isleaf, 0, sizeof(isleaf));
	memset(cntnonleaf, 0, sizeof(cntnonleaf));

	buildtree(0);

	freopen("tree2.out", "w", stdout);

	for (i = 1; i < N; i++)
		for (j = i + 1; j <= N; j++)
			if (T[i][j])
				printf("%d %d\n", i, j);

	return 0;
}
